@@ -35,6 +35,7 @@ import org.perl6.nqp.sixmodel.reprs.ReentrantMutex;
 import org.perl6.nqp.sixmodel.reprs.Semaphore;
 import org.perl6.nqp.sixmodel.reprs.ConcBlockingQueue;
 import org.perl6.nqp.sixmodel.reprs.ConditionVariable;
+import org.perl6.nqp.sixmodel.reprs.AsyncTask;
 
 public class REPRRegistry {
     private static HashMap<String, Integer> reprIdMap = new HashMap<String, Integer>();
@@ -94,5 +95,6 @@ public class REPRRegistry {
         addREPR("Semaphore", new Semaphore());
         addREPR("ConcBlockingQueue", new ConcBlockingQueue());
         addREPR("ConditionVariable", new ConditionVariable());
+        addREPR("AsyncTask", new AsyncTask());
     }
 }
